#include "lmbf.h"
#include "fftplot.h"
// cumbia
#include <cumbiapool.h>
#include <cuserviceprovider.h>
#include <cumacros.h>
#include <quapps.h>
#include <quplotcurve.h>
// cumbia

#include <qwt_plot_item.h>
#include "ui_lmbf.h"
#include <QDebug>

LmbfGui::LmbfGui(CumbiaPool *cumbia_pool, QWidget *parent) :
    QWidget(parent) {
    // cumbia
    CuModuleLoader mloader(cumbia_pool, &m_ctrl_factory_pool, &m_log_impl);
    cu_pool = cumbia_pool;
    ui = new Ui::LmbfGui;
    ui->setupUi(this, cu_pool, m_ctrl_factory_pool);


    // plot options
    ui->tPlotLightMarkerADC->setOptions(CuData("period", 500));
    ui->fftplot->setOptions(CuData("period", 500));
    ui->tPlotLightMarkerADC->setSource("$3/I_ADC_MMS_DELTA");
    ui->fftplot->setSource("lmbf/buf/0/FFTADC0Avg");

//    fftScale = new fftScaleDraw();
//    ui->fftplot->setAxisScaleDraw(QwtPlot::xBottom, fftScale);
    ui->fftplot->axisScaleEngine(QwtPlot::xBottom)->setBase(2);

    num_turns=1024; //fake value for starting

    normalpalette = ui->tLabel_7->palette();
    alarmpalette  = ui->tLabel_7->palette();
    normalpalette.setColor(QPalette::Background, QColor(Qt::green));
    alarmpalette.setColor(QPalette::Background, QColor(Qt::red));

    turns_reader = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
    turns_reader->attach(this, SLOT(update_num_turns(double)));
    turns_reader->setSource("lmbf/buf/0/numturns");

    sevr_reader = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
    sevr_reader->attach(this, SLOT(update_sevr(QString)));
    sevr_reader->setSource("$3/IQ_STA_SEVR");
    
    QwtText au=QwtText("Amplitude [a.u]");
    au.setFont(QFont("Helvetica",9));

    ui->fftplot->setAxisTitle(QwtPlot::yLeft, au);
    ui->tPlotLightMarkerADC->setAxisTitle(QwtPlot::yLeft, au);
    ui->tPlotLightMarkerModesUp->setAxisTitle(QwtPlot::yLeft, au);
    ui->tPlotLightMarkerModesDown->setAxisTitle(QwtPlot::yLeft, au);
    
    QwtText instab=QwtText("Bunch instability");
    instab.setFont(QFont("Helvetica",10));
    ui->tPlotLightMarkerADC->setTitle(instab);
    
    QwtText spectr=QwtText("Average bunch spectrum");
    spectr.setFont(QFont("Helvetica",10));
    ui->fftplot->setTitle(spectr);
    
    QwtText lmodes=QwtText("Longitudinal modes");
    lmodes.setFont(QFont("Helvetica",10));
    ui->tPlotLightMarkerModesUp->setTitle(lmodes);
    
    QwtText modelegend=QwtText("mode number");
    modelegend.setFont(QFont("Helvetica",9));
    ui->tPlotLightMarkerModesDown->setAxisTitle(QwtPlot::xBottom, modelegend);
    
    QwtText bnum=QwtText("bunch number");
    bnum.setFont(QFont("Helvetica",9));
    ui->tPlotLightMarkerADC->setAxisTitle(QwtPlot::xBottom, bnum);
    
    QwtText fractune=QwtText("fractional tune");
    fractune.setFont(QFont("Helvetica",9));
    ui->fftplot->setAxisTitle(QwtPlot::xBottom, fractune);

    ui->fftplot->setAxisLabelRotation(QwtPlot::xBottom, 30);
    ui->fftplot->setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignRight);

    connect(ui->tPlotLightMarkerADC, SIGNAL(curveAdded(QuPlotCurve *)), this, SLOT(setFillArea(QuPlotCurve *)));
    connect(ui->tPlotLightMarkerModesUp, SIGNAL(curveAdded(QuPlotCurve *)), this, SLOT(setCurveSticksStyle(QuPlotCurve *)));
    connect(ui->tPlotLightMarkerModesDown, SIGNAL(curveAdded(QuPlotCurve *)), this, SLOT(setCurveSticksStyle(QuPlotCurve *)));

    // scale change
    foreach(QPushButton *pb, findChildren<QPushButton *>(QRegularExpression("pbscale.*")))
        connect(pb, SIGNAL(clicked()),this , SLOT(scale_change()));

// use cumbia 1.3.1 LmbfPlotMarkerFormatter to remove device names from source
#if CUMBIA_QTCONTROLS_VERSION >= 0x010301
    foreach(QuPlotBase *p, findChildren<QuPlotBase *>()) {
        QuPlotMarkerComponent *mk = static_cast<QuPlotMarkerComponent *>(p->getComponent("marker"));
        mk->setFormatter(new LmbfPlotMarkerFormatter);
    }
#endif
} 

LmbfGui::~LmbfGui()
{
    delete turns_reader;
    delete ui;
}

void LmbfGui::update_num_turns(double nturns)
{
    qDebug() << "update_num_turns " << nturns ;
    ui->fftplot->set_num_turns(nturns);
    if (nturns > 0 && nturns != num_turns){
        num_turns = nturns;
        int bound= static_cast<int>(num_turns/20);
        ui->fftplot->setXUpperBound( bound );
        ui->fftplot->setAxisScale(QwtPlot::xBottom, 0.0, bound, 0);
        ui->fftplot->axisScaleEngine(QwtPlot::xBottom)->setBase(2);
    }
}




void LmbfGui::update_sevr(QString sevr)
{
    if (sevr == "NO_ALARM"){
        ui->tLabel_7->setPalette(normalpalette);
        ui->tLabel_7->setText("OK");
    }
    else{
        ui->tLabel_7->setPalette(alarmpalette);
        ui->tLabel_7->setText("NOT OK");
    }
}

void LmbfGui::setCurveSticksStyle(QuPlotCurve *c) {
    QPen p = c->pen();
    p.setWidthF(2.0);
    c->setPen(p);
    c->setStyle(QwtPlotCurve::Sticks);
}

void LmbfGui::setFillArea(QuPlotCurve *c) {
    c->setBrush(QBrush(c->pen().color().lighter()));
    c->setBaseline(0.0);
}

void LmbfGui::scale_change() {
    QuPlotBase *p = ui->tabWidget->findChild<QuPlotBase *>(sender()->property("plot").toString());
    const QString& s = sender()->objectName();
    bool up = s.endsWith("Up", Qt::CaseInsensitive);
    double ub = p->yUpperBound();
    // assume lower bound zero
    up ? ub = 2 * ub : ub = ub / 2.0;
    p->setYUpperBound(ub);
}

#if CUMBIA_QTCONTROLS_VERSION >= 0x010301
QString LmbfPlotMarkerFormatter::format(const QuPlotBase *plot,
                                        const QList<QwtPlotCurve *> curves,
                                        const int index, QString& xt, QString& yt) const {
    QString s;
    if(curves.size() == 0)
        return s;
    double x = curves[0]->data()->sample(index).x();
    /* place the x coordinate label, taken from the x axis scale */
    const QwtScaleDraw *scaleDraw = plot->axisScaleDraw(curves[0]->xAxis());
    xt = QString("x: %1").arg(scaleDraw->label(x).text());
    for(int i = 0; i < curves.size(); i++) {
        const QwtPlotCurve *pc = curves[i];
        const double &y1 = pc->data()->sample(index).y();
        /* place the curve title on the marker text first */
        QString nam = pc->title().text().section('/', -1); // take att name only
        s += QString("%1: %2").arg(nam).arg(y1, 0, 'g', 5);
        if(i < curves.size() - 1)
            s+= "\n";
    }
    yt = s;
    return xt + "\n" + s;
}

#endif

